
// NumberTests.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, April 2023

/* A variety of math operations, separated into four
   groups:
     * BigDecimal Pitfalls
     * Constants
     * Nut Functions  (Nut is my utility class)
       with a few JML functions included
     * Big-Math tests

   Usage:
     > compile NumberTests.java
     > run NumberTests

 */

import java.math.*;
import java.util.*;

import de.tilman_neumann.jml.roots.SqrtInt;
import de.tilman_neumann.jml.transcendental.Ln;
import de.tilman_neumann.jml.transcendental.Exp;
import de.tilman_neumann.jml.precision.Precision;

import ch.obermuhlner.math.big.BigDecimalMath;


public class NumberTests
{

  public static void main(String[] args)
  {
    System.out.println("--- Pitfalls ---");
    // https://blogs.oracle.com/javamagazine/post/four-common-pitfalls-of-the-bigdecimal-class-and-how-to-avoid-them
    System.out.println("0.1 as constructor double =\n" + new BigDecimal(0.1));
    System.out.println("0.1 as constructor string = " + new BigDecimal("0.1"));
    System.out.println("0.1 as valueOf() = " + BigDecimal.valueOf(0.1));


    System.out.println("\nEquality of 1 and 1.0 = " + 
                new BigDecimal("1").equals(new BigDecimal("1.0")));

    System.out.println("Comparison of 1 and 1.0 = " + 
                ((new BigDecimal("1").compareTo(new BigDecimal("1.0"))) == 0));

    System.out.println("\nString vs. valueOf(): " + 
              new BigDecimal("1.01234567890123456789") + " vs. " + 
              BigDecimal.valueOf(1.01234567890123456789) );

    System.out.println("\n--- Constants ---");
	System.out.println("Math.PI: " + Math.PI);
	System.out.println("Math.E: " + Math.E);
	System.out.println("Math.sqrt(2): " + Math.sqrt(2));
	System.out.println("Phi = (1 + Math.sqrt(5))/2: " + (1.0 + Math.sqrt(5))/2.0);

	System.out.println("\nNut.PI: " + Nut.PI);
	System.out.println("Nut.E: " + Nut.E);
	System.out.println("Nut.SQRT2: " + Nut.SQRT2);
	System.out.println("Nut.PHI: " + Nut.PHI);


    System.out.println("\n--- Nut Functions ---");

    System.out.println("257 in base 2 = " + Arrays.toString(Nut.toBase(257, 2)));

    System.out.println("\ncatalan(7) = " + Nut.catalan(7));
    System.out.println("isPerfect(6)? " + Nut.isPerfect(6));   // true
    System.out.println("isArmstrong(371)? " + Nut.isArmstrong(371));   // true
    
    System.out.println("\nlog3(81) = " + Nut.log(3, 81));  // 4
    System.out.println("log3(81000000000000) = " + 
              Nut.log(3, new BigDecimal("81000000000000"), 10));   // about 30
    
    System.out.println("\n(3.11)^(2.02) = " + 
              Nut.pow( BigDecimal.valueOf(3.11), BigDecimal.valueOf(2.02), 8));

    System.out.println("\nGardner's e^(pi*sqrt(163)) = \n  " +     // Martin Gardner, SciAm, April 1975
      Exp.exp( Nut.PI.multiply( 
               BigDecimal.valueOf(163).sqrt( MathContext.DECIMAL128)),
               Precision.valueOf(40)) );

    BigDecimal left1 = BigDecimal.valueOf(3987).pow(12);
    BigDecimal left2 = BigDecimal.valueOf(4365).pow(12);
    BigDecimal right = BigDecimal.valueOf(4472).pow(12);
    BigDecimal simpsonErr = left1.add(left2).subtract(right).divide(right, MathContext.DECIMAL64);
    System.out.println("\nSimpson's Fermat 3987^12 + 4365^12 = 4472^12\n  Error: " + simpsonErr);
             
    double sig = Nut.sigmoid(5);
    System.out.println("\nsigmoid(5) = " + sig);
    System.out.println("invSigmoid() = " + Nut.invSigmoid(sig));  // 5


    BigInteger f100 = de.tilman_neumann.jml.combinatorics.Factorial.factorial(100);   
                        // to avoid clash with Common Apache Numbers
    System.out.println("\nfact(100) = \n  " + f100);
        
    BigInteger s100 = Nut.stirling(100);
    System.out.println("\nStirling's fact(100) = \n  " +  s100);

    BigDecimal err = new BigDecimal(f100.subtract(s100).multiply(BigInteger.valueOf(100))).
                           divide(new BigDecimal(f100), MathContext.DECIMAL64);
    System.out.println("\n% Error between two fact(100)'s = " +  err);


    BigDecimal big = new BigDecimal("100000000000000000000");
    System.out.println("\nBigDecimal sqrt(" + big
                           + ") = " + big.sqrt(new MathContext(10)));  // 1.0e10

    BigInteger[] luRes = SqrtInt.iSqrt(f100);
    System.out.println("\nInteger sqroot of fact(100) = \n   " + luRes[0] + " -- " + luRes[1]);

    testBigMath();
  }  // end of main()


  private static void testBigMath()
  {
    System.out.println("\n --- Big-Math Tests ---");

    MathContext mc = new MathContext(100);
    System.out.println("Using a precision of " + mc.getPrecision() + " digits");

    BigDecimal two = BigDecimal.valueOf(2);
    BigDecimal tenth = BigDecimal.valueOf(0.1);

    System.out.println();
    System.out.println("BigDecimal Functions:");
    System.out.println("  sqrt(2) = " + BigDecimalMath.sqrt(two, mc));
    System.out.println("  root(2, 3) = " +
                       BigDecimalMath.root(two, BigDecimal.valueOf(3), mc));
    System.out.println("  pow(2, 3) = " +
                       BigDecimalMath.pow(two, BigDecimal.valueOf(3), mc));
    System.out.println("  pow(2.1, 3.4) = " +
                       BigDecimalMath.pow(BigDecimal.valueOf(2.1),
                                          BigDecimal.valueOf(3.4), mc));
    System.out.println("  log(2) = " + BigDecimalMath.log(two, mc));
    System.out.println("  log2(2) = " + BigDecimalMath.log2(two, mc));
    System.out.println("  log10(2) = " + BigDecimalMath.log10(two, mc));
    System.out.println("  exp(2) = " + BigDecimalMath.exp(two, mc));
    System.out.println("  sin(2) = " + BigDecimalMath.sin(two, mc));
    System.out.println("  cos(2) = " + BigDecimalMath.cos(two, mc));
    System.out.println("  tan(2) = " + BigDecimalMath.tan(two, mc));
    System.out.println("  cot(2) = " + BigDecimalMath.cot(two, mc));
    System.out.println("  asin(0.1) = " + BigDecimalMath.asin(tenth, mc));
    System.out.println("  acos(0.1) = " + BigDecimalMath.acos(tenth, mc));
    System.out.println("  atan(0.1) = " + BigDecimalMath.atan(tenth, mc));
    System.out.println("  acot(0.1) = " + BigDecimalMath.acot(tenth, mc));
    System.out.println("  sinh(2) = " + BigDecimalMath.sinh(two, mc));
    System.out.println("  cosh(2) = " + BigDecimalMath.cosh(two, mc));
    System.out.println("  tanh(2) = " + BigDecimalMath.tanh(two, mc));
    System.out.println("  asinh(0.1) = " + BigDecimalMath.asinh(tenth, mc));
    System.out.println("  acosh(2)   = " + BigDecimalMath.acosh(two, mc));
    System.out.println("  atanh(0.1) = " + BigDecimalMath.atanh(tenth, mc));
    System.out.println("  factorial(6)= " + BigDecimalMath.factorial(6));
    System.out.println();

    System.out.println("Constants:");
    System.out.println("  pi = " + BigDecimalMath.pi(mc));
    System.out.println("  e  = " + BigDecimalMath.e(mc));
    System.out.println();

    System.out.println("Methods:");
    System.out.println("  mantissa(1.456E99) = " +
             BigDecimalMath.mantissa(BigDecimal.valueOf(1.456E99)));
    System.out.println("  exponent(1.456E99) = " +
             BigDecimalMath.exponent(BigDecimal.valueOf(1.456E99)));
    System.out.println("  integralPart(123.456) = " +
             BigDecimalMath.integralPart(BigDecimal.valueOf(123.456)));
    System.out.println("  fractionalPart(123.456) = " +
             BigDecimalMath.fractionalPart(BigDecimal.valueOf(123.456)));
    System.out.println("  isIntValue(123) = " +
             BigDecimalMath.isIntValue(BigDecimal.valueOf(123)));
    System.out.println("  isIntValue(123.456) = " +
             BigDecimalMath.isIntValue(BigDecimal.valueOf(123.456)));
  } /* testBigMath */


}  // end of NumberTests class

